@@ -6,12 +6,31 @@ from srxraylib.sources import srfunc
 from srxraylib.util.h5_simple_writer import H5SimpleWriter
 
 from scipy.interpolate import interp1d
-from scipy.integrate import cumtrapz
+try:                from scipy.integrate import cumtrapz
+except ImportError: from scipy.integrate import cumulative_trapezoid as cumtrapz
 import scipy.constants as codata
 
 from xoppylib.fit_gaussian2d import fit_gaussian2d, info_params, twoD_Gaussian
 
-from oasys.util.oasys_util import get_fwhm
+# --------------------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------
+
+# copied from OASYS1
+def get_fwhm(histogram, bins, ret0=None):
+    fwhm = ret0
+    quote = ret0
+    coordinates = None
+
+    if histogram.size > 1:
+        quote = numpy.max(histogram)*0.5
+        cursor = numpy.where(histogram >= quote)
+
+        if histogram[cursor].size > 1:
+            bin_size    = bins[1]-bins[0]
+            fwhm        = bin_size*(cursor[0][-1]-cursor[0][0])
+            coordinates = (bins[cursor[0][0]], bins[cursor[0][-1]])
+
+    return fwhm, quote, coordinates
 
 # --------------------------------------------------------------------------------------------
 # --------------------------------------------------------------------------------------------
@@ -23,15 +42,17 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
                   PSI_MRAD_PLOT=1.0,PSI_MIN=-1.0,PSI_MAX=1.0,PSI_NPOINTS=500,TYPE_CALC=0,FILE_DUMP=0):
 
     # electron energy in GeV
-    gamma = BEAM_ENERGY_GEV*1e3 / srfunc.codata_mee
+    codata_mee = 1e-6 * codata.m_e * codata.c**2 / codata.e  # electron mass in meV
+    m2ev = codata.c * codata.h / codata.e
+    gamma = BEAM_ENERGY_GEV * 1e3 / codata_mee
 
     r_m = MACHINE_R_M      # magnetic radius in m
     if RB_CHOICE == 1:
-        r_m = srfunc.codata_me * srfunc.codata_c / srfunc.codata_ec / BFIELD_T * numpy.sqrt(gamma * gamma - 1)
+        r_m = codata.m_e * codata.c / codata.e / BFIELD_T * numpy.sqrt(gamma * gamma - 1)
 
     # calculate critical energy in eV
     ec_m = 4.0*numpy.pi*r_m/3.0/numpy.power(gamma,3) # wavelength in m
-    ec_ev = srfunc.m2ev / ec_m
+    ec_ev = m2ev / ec_m
 
     print("\nLorent factor gamma: %g" % (gamma))
     print("\nMagnetic radius: %g m" % (r_m))
@@ -75,7 +96,7 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
 
         a6=numpy.zeros((8,len(energy_ev)))
         a1 = energy_ev
-        spectral_power = numpy.array(a5)*1e3 * srfunc.codata_ec
+        spectral_power = numpy.array(a5) * 1e3 * codata.e
 
         # calculate cumulated power
         if VER_DIV in [0,2]:
@@ -93,7 +114,7 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
             print("\nPower from integral of spectrum (sum rule, WARNING: energy in LOG): %8.3f %s" % (cumulated_power[-1], cumulated_power_unit))
 
         a6[0, :] = (a1)
-        a6[1, :] = srfunc.m2ev * 1e10 / (a1)
+        a6[1, :] = m2ev * 1e10 / (a1)
         a6[2, :] = (a1)/ec_ev # E/Ec
         a6[3, :] = numpy.array(a5par)/numpy.array(a5)
         a6[4, :] = numpy.array(a5per)/numpy.array(a5)
@@ -126,7 +147,7 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
         tmp = srfunc.sync_ang(1, angle_mrad, energy=PHOT_ENERGY_MIN, i_a=CURRENT_A, hdiv_mrad=HOR_DIV_MRAD, e_gev=BEAM_ENERGY_GEV, ec_ev=ec_ev)
         tmp.shape = -1
         a6[5,:] = tmp
-        a6[6,:] = a6[5,:] * srfunc.codata_ec * 1e3
+        a6[6,:] = a6[5,:] * codata.e * 1e3
 
         coltitles=['Psi[mrad]','Psi[rad]*Gamma','F','F s-pol','F p-pol','Flux[Ph/sec/0.1%bw/mrad(Psi)]','Power[Watts/eV/mrad(Psi)]']
 
@@ -151,7 +172,7 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
                 ij += 1
                 a6[0,ij] = a[i]
                 a6[1,ij] = energy_ev[j]
-                a6[2,ij] = fm[i,j] * srfunc.codata_ec * 1e3
+                a6[2,ij] = fm[i,j] * codata.e * 1e3
                 a6[3,ij] = fm[i,j]
 
         coltitles=['Psi [mrad]','Photon Energy [eV]','Power [Watts/eV/mrad(Psi)]','Flux [Ph/sec/0.1%bw/mrad(Psi)]']
@@ -179,7 +200,7 @@ def xoppy_calc_bm(MACHINE_NAME="ESRF bending magnet",RB_CHOICE=0,MACHINE_R_M=25.
 
     # if TYPE_CALC == 0:
     #     if LOG_CHOICE == 0:
-    #         print("\nPower from integral of spectrum: %15.3f W"%(a5.sum() * 1e3*srfunc.codata_ec * (energy_ev[1]-energy_ev[0])))
+    #         print("\nPower from integral of spectrum: %15.3f W"%(a5.sum() * 1e3*codata.e * (energy_ev[1]-energy_ev[0])))
 
     return a6.T, fm, a, energy_ev
 
@@ -206,9 +227,9 @@ def xoppy_calc_wigg(FIELD=0,NPERIODS=12,ULAMBDA=0.125,K=14.0,ENERGY=6.04,PHOT_EN
     if FIELD == 2:
         # magnetic field from harmonics
         # hh = srfunc.wiggler_harmonics(b_t,Nh=41,fileOutH="tmp.h")
-        t0,p = srfunc.wiggler_trajectory(b_from=2, nPer=NPERIODS, nTrajPoints=NTRAJPOINTS, \
+        t0, p = srfunc.wiggler_trajectory(b_from=2, nPer=NPERIODS, nTrajPoints=NTRAJPOINTS, \
                                          ener_gev=ENERGY, per=ULAMBDA, inData="", trajFile=outFileTraj)
-    print(p)
+
     #
     # now spectra
     #
@@ -223,6 +244,84 @@ def xoppy_calc_wigg(FIELD=0,NPERIODS=12,ULAMBDA=0.125,K=14.0,ENERGY=6.04,PHOT_EN
     print("\nPower from integral of spectrum (sum rule): %8.3f W" % (cumulated_power[-1]))
     return e, f0, p0 , cumulated_power, t0, p
 
+# --------------------------------------------------------------------------------------------
+# --------------------------------------------------------------------------------------------
+
+def xoppy_calc_wiggler_on_aperture(FIELD=0,
+                                   NPERIODS=12,
+                                   ULAMBDA=0.125,
+                                   K=14.0,
+                                   ENERGY=6.04,
+                                   PHOT_ENERGY_MIN=100.0,
+                                   PHOT_ENERGY_MAX=100100.0,
+                                   NPOINTS=100,
+                                   NTRAJPOINTS=101,
+                                   CURRENT=200.0,
+                                   FILE="?",
+                                   SLIT_FLAG=0,
+                                   SLIT_D=30.0,
+                                   SLIT_NY=100,
+                                   SLIT_WIDTH_H_MM=1.0,
+                                   SLIT_HEIGHT_V_MM=1.0,
+                                   SLIT_CENTER_H_MM=0.0,
+                                   SLIT_CENTER_V_MM=0.0,
+                                   SHIFT_X_FLAG=0,
+                                   SHIFT_X_VALUE=0.0,
+                                   SHIFT_BETAX_FLAG=0,
+                                   SHIFT_BETAX_VALUE=0.0,
+                                   TRAJ_RESAMPLING_FACTOR=1e4,
+                                   SLIT_POINTS_FACTOR=1,
+                                   ):
+
+    print("Inside xoppy_calc_wiggler_on_aperture. ")
+
+    outFileTraj = "xwiggler_traj.spec"
+    outFile = "xwiggler.spec"
+
+    if FIELD == 0:
+        t0,p = srfunc.wiggler_trajectory(b_from=0, nPer=NPERIODS, nTrajPoints=NTRAJPOINTS, \
+                                         ener_gev=ENERGY, per=ULAMBDA, kValue=K, \
+                                         trajFile=outFileTraj)
+    if FIELD == 1:
+        # magnetic field from B(s) map
+        t0,p = srfunc.wiggler_trajectory(b_from=1, nPer=NPERIODS, nTrajPoints=NTRAJPOINTS, \
+                                         ener_gev=ENERGY, inData=FILE, trajFile=outFileTraj,\
+                                         shift_x_flag = SHIFT_X_FLAG, \
+                                         shift_x_value = SHIFT_X_VALUE, \
+                                         shift_betax_flag = SHIFT_BETAX_FLAG, \
+                                         shift_betax_value = SHIFT_BETAX_VALUE)
+    if FIELD == 2:
+        # magnetic field from harmonics
+        # hh = srfunc.wiggler_harmonics(b_t,Nh=41,fileOutH="tmp.h")
+        t0, p = srfunc.wiggler_trajectory(b_from=2, nPer=NPERIODS, nTrajPoints=NTRAJPOINTS, \
+                                         ener_gev=ENERGY, per=ULAMBDA, inData="", trajFile=outFileTraj)
+
+    #
+    # now spectra
+    #
+    if SLIT_FLAG == 0:
+        e, f0, p0 = srfunc.wiggler_spectrum(t0, enerMin=PHOT_ENERGY_MIN, enerMax=PHOT_ENERGY_MAX, nPoints=NPOINTS, \
+                                        electronCurrent=CURRENT*1e-3, outFile=outFile, elliptical=False)
+    else:
+        e, f0, p0 = srfunc.wiggler_spectrum_on_aperture(t0, enerMin=PHOT_ENERGY_MIN, enerMax=PHOT_ENERGY_MAX, nPoints=NPOINTS, \
+                                            electronCurrent=CURRENT * 1e-3, outFile=outFile, elliptical=False,
+                                            psi_min=(-SLIT_HEIGHT_V_MM / 2 + SLIT_CENTER_V_MM) / SLIT_D, # mrad
+                                            psi_max=(SLIT_HEIGHT_V_MM / 2 + SLIT_CENTER_V_MM) / SLIT_D,  # mrad
+                                            psi_npoints=SLIT_NY,
+                                            theta_min=(-SLIT_WIDTH_H_MM / 2 + SLIT_CENTER_H_MM) / SLIT_D,  # mrad
+                                            theta_max=(SLIT_WIDTH_H_MM / 2 + SLIT_CENTER_H_MM) / SLIT_D,  # mrad
+                                            traj_res_fac=TRAJ_RESAMPLING_FACTOR,
+                                            slit_points_factor=SLIT_POINTS_FACTOR
+                                            )
+
+    try:
+        cumulated_power = p0.cumsum() * numpy.abs(e[0] - e[1])
+    except:
+        cumulated_power = 0.0
+
+    print("\n Flux peak: %.2e Ph/s/0.1%%BW at the photon energy: %.2f eV" % (max(f0), e[numpy.argmax(f0)]))    
+    print("\nPower from integral of spectrum (sum rule): %8.3f W" % (cumulated_power[-1]))
+    return e, f0, p0 , cumulated_power, t0, p
 
 def trapezoidal_rule_2d_1darrays(data2D,h=None,v=None):
     if h is None:
@@ -294,7 +393,7 @@ def xoppy_calc_wiggler_radiation(
             shift_x_flag = SHIFT_X_FLAG,
             shift_x_value = SHIFT_X_VALUE,
             shift_betax_flag = SHIFT_BETAX_FLAG,
-            shift_betax_value = SHIFT_BETAX_FLAG)
+            shift_betax_value = SHIFT_BETAX_VALUE)
     if FIELD == 2:
         raise("Not implemented")
 
@@ -580,135 +679,141 @@ if __name__ == "__main__":
     set_qt()
 
 
-    # e, h, v, p, traj = xoppy_calc_wiggler_radiation(PHOTONENERGYPOINTS=100,do_plot = False, POLARIZATION=0, NPERIODS=3.5)
-    # e, h, v, p, traj = xoppy_calc_wiggler_radiation(PHOTONENERGYPOINTS=3,FIELD=1)
+    if True:
 
-    # create_magnetic_field_for_bending_magnet(do_plot=True, filename="tmp.txt", B0=-1.0, divergence=1e-3, radius=10.0,
-    #                                         npoints=500)
+        # script to make the calculations (created by XOPPY:wiggler)
+
+        from xoppylib.sources.xoppy_bm_wiggler import xoppy_calc_wigg
+
+        energy, flux, spectral_power, cumulated_power, traj, traj_info = xoppy_calc_wigg(
+            FIELD=0,
+            NPERIODS=10,
+            ULAMBDA=0.15,
+            K=22.591,
+            ENERGY=6.0,
+            PHOT_ENERGY_MIN=100.0,
+            PHOT_ENERGY_MAX=100100.0,
+            NPOINTS=100,
+            NTRAJPOINTS=101,
+            CURRENT=200.0,
+            FILE="?")
+
+        #
+        # script to make the calculations (created by XOPPY:WS)
+        #
+        import numpy
+        from xoppylib.xoppy_run_binaries import xoppy_calc_ws
+
+        out_file = xoppy_calc_ws(
+            ENERGY=6.0,
+            CUR=200.0,
+            PERIOD=15.0,
+            N=10.0,
+            KX=0.0,
+            KY=22.591,
+            EMIN=100.0,
+            EMAX=100100.0,
+            NEE=1000,
+            D=30.0,
+            XPC=0.0,
+            YPC=0.0,
+            XPS=115.0,
+            YPS=60.0,
+            NXP=70,
+            NYP=70,
+        )
+
+        # data to pass to power
+        data = numpy.loadtxt(out_file)
+        energy2 = data[:, 0]
+        flux2 = data[:, 1]
+        # spectral_power = data[:, 2]
+        # cumulated_power = data[:, 3]
+
+        #
+        # example plot
+        #
+        from srxraylib.plot.gol import plot
+
+        plot(energy, flux,
+             energy2, flux2,
+             xtitle="Photon energy [eV]", ytitle="Flux [photons/s/0.1%bw]", title="WS Flux",
+             legend=['WIGGLER', 'WS'],
+             xlog=0, ylog=0, show=True)
+
+
+    ##########################################################################
+    # wiggler comparison (integrated over a slit)
     #
-    # e, h, v, p, traj = xoppy_calc_wiggler_radiation(PHOTONENERGYPOINTS=3, do_plot=True, POLARIZATION=0,
-    #                                                 FIELD=1, FILE="tmp.txt")
 
     #
-    # script to make the calculations (created by XOPPY:wiggler_radiation)
-    #
-    h5_parameters = dict()
-    h5_parameters["ELECTRONENERGY"] = 6.0
-    h5_parameters["ELECTRONCURRENT"] = 0.2
-    h5_parameters["PERIODID"] = 0.15
-    h5_parameters["NPERIODS"] = 10.0
-    h5_parameters["KV"] = 21.015
-    h5_parameters["FIELD"] = 0  # 0= sinusoidal, 1=from file
-    h5_parameters["FILE"] = ''
-    h5_parameters["POLARIZATION"] = 0  # 0=total, 1=s, 2=p
-    h5_parameters["DISTANCE"] = 30.0
-    h5_parameters["HSLITPOINTS"] = 500
-    h5_parameters["VSLITPOINTS"] = 500
-    h5_parameters["PHOTONENERGYMIN"] = 100.0
-    h5_parameters["PHOTONENERGYMAX"] = 100100.0
-    h5_parameters["PHOTONENERGYPOINTS"] = 101
-    h5_parameters["SHIFT_X_FLAG"] = 0
-    h5_parameters["SHIFT_X_VALUE"] = 0.0
-    h5_parameters["SHIFT_BETAX_FLAG"] = 0
-    h5_parameters["SHIFT_BETAX_VALUE"] = 0.0
-    h5_parameters["CONVOLUTION"] = 1
-
-    e, h, v, p, traj = xoppy_calc_wiggler_radiation(
-        ELECTRONENERGY=h5_parameters["ELECTRONENERGY"],
-        ELECTRONCURRENT=h5_parameters["ELECTRONCURRENT"],
-        PERIODID=h5_parameters["PERIODID"],
-        NPERIODS=h5_parameters["NPERIODS"],
-        KV=h5_parameters["KV"],
-        FIELD=h5_parameters["FIELD"],
-        FILE=h5_parameters["FILE"],
-        POLARIZATION=h5_parameters["POLARIZATION"],
-        DISTANCE=h5_parameters["DISTANCE"],
-        HSLITPOINTS=h5_parameters["HSLITPOINTS"],
-        VSLITPOINTS=h5_parameters["VSLITPOINTS"],
-        PHOTONENERGYMIN=h5_parameters["PHOTONENERGYMIN"],
-        PHOTONENERGYMAX=h5_parameters["PHOTONENERGYMAX"],
-        PHOTONENERGYPOINTS=h5_parameters["PHOTONENERGYPOINTS"],
-        SHIFT_X_FLAG=h5_parameters["SHIFT_X_FLAG"],
-        SHIFT_X_VALUE=h5_parameters["SHIFT_X_VALUE"],
-        SHIFT_BETAX_FLAG=h5_parameters["SHIFT_BETAX_FLAG"],
-        SHIFT_BETAX_VALUE=h5_parameters["SHIFT_BETAX_VALUE"],
-        CONVOLUTION=h5_parameters["CONVOLUTION"],
-        h5_file="wiggler_radiation.h5",
-        h5_entry_name="XOPPY_RADIATION",
-        h5_initialize=True,
-        h5_parameters=h5_parameters,
-        do_plot=0,
-        PASSEPARTOUT=1,
-    )
-
-    # example plot
-    from srxraylib.plot.gol import plot_image
-
-    plot_image(p[0], h, v, title="Flux [photons/s] per 0.1 bw per mm2 at %9.3f eV" % (25100.0), xtitle="H [mm]",
-               ytitle="V [mm]")
-    #
-    # end script
+    # script to make the calculations (created by XOPPY:wiggler)
     #
 
-    # #
-    # # script to make the calculations (created by XOPPY:wiggler_radiation)
-    # #
-    #
-    # from xoppylib.xoppy_bm_wiggler import xoppy_calc_wiggler_radiation
-    #
-    # h5_parameters = dict()
-    # h5_parameters["ELECTRONENERGY"] = 6.0
-    # h5_parameters["ELECTRONCURRENT"] = 0.2
-    # h5_parameters["PERIODID"] = 0.15
-    # h5_parameters["NPERIODS"] = 10.0
-    # h5_parameters["KV"] = 21.015
-    # h5_parameters["FIELD"] = 0  # 0= sinusoidal, 1=from file
-    # h5_parameters["FILE"] = ''
-    # h5_parameters["POLARIZATION"] = 0  # 0=total, 1=s, 2=p
-    # h5_parameters["DISTANCE"] = 30.0
-    # h5_parameters["HSLITPOINTS"] = 500
-    # h5_parameters["VSLITPOINTS"] = 500
-    # h5_parameters["PHOTONENERGYMIN"] = 100.0
-    # h5_parameters["PHOTONENERGYMAX"] = 100100.0
-    # h5_parameters["PHOTONENERGYPOINTS"] = 11
-    # h5_parameters["SHIFT_X_FLAG"] = 0
-    # h5_parameters["SHIFT_X_VALUE"] = 0.0
-    # h5_parameters["SHIFT_BETAX_FLAG"] = 0
-    # h5_parameters["SHIFT_BETAX_VALUE"] = 0.0
-    # h5_parameters["CONVOLUTION"] = 1
-    #
-    # e, h, v, p, traj = xoppy_calc_wiggler_radiation(
-    #     ELECTRONENERGY=h5_parameters["ELECTRONENERGY"],
-    #     ELECTRONCURRENT=h5_parameters["ELECTRONCURRENT"],
-    #     PERIODID=h5_parameters["PERIODID"],
-    #     NPERIODS=h5_parameters["NPERIODS"],
-    #     KV=h5_parameters["KV"],
-    #     FIELD=h5_parameters["FIELD"],
-    #     FILE=h5_parameters["FILE"],
-    #     POLARIZATION=h5_parameters["POLARIZATION"],
-    #     DISTANCE=h5_parameters["DISTANCE"],
-    #     HSLITPOINTS=h5_parameters["HSLITPOINTS"],
-    #     VSLITPOINTS=h5_parameters["VSLITPOINTS"],
-    #     PHOTONENERGYMIN=h5_parameters["PHOTONENERGYMIN"],
-    #     PHOTONENERGYMAX=h5_parameters["PHOTONENERGYMAX"],
-    #     PHOTONENERGYPOINTS=h5_parameters["PHOTONENERGYPOINTS"],
-    #     SHIFT_X_FLAG=h5_parameters["SHIFT_X_FLAG"],
-    #     SHIFT_X_VALUE=h5_parameters["SHIFT_X_VALUE"],
-    #     SHIFT_BETAX_FLAG=h5_parameters["SHIFT_BETAX_FLAG"],
-    #     SHIFT_BETAX_VALUE=h5_parameters["SHIFT_BETAX_VALUE"],
-    #     CONVOLUTION=h5_parameters["CONVOLUTION"],
-    #     h5_file="wiggler_radiation.h5",
-    #     h5_entry_name="XOPPY_RADIATION",
-    #     h5_initialize=True,
-    #     h5_parameters=h5_parameters,
-    #     do_plot=1,
-    # )
-    #
-    # # example plot
-    # from srxraylib.plot.gol import plot_image
-    #
-    # plot_image(p[0], h, v, title="Flux [photons/s] per 0.1 bw per mm2 at %9.3f eV" % (100.0), xtitle="H [mm]",
-    #            ytitle="V [mm]")
-    #
-    # end script
-    #
+    if True:
+
+        from xoppylib.sources.xoppy_bm_wiggler import xoppy_calc_wiggler_on_aperture
+
+        energy, flux, spectral_power, cumulated_power, traj, traj_info = xoppy_calc_wiggler_on_aperture(
+            FIELD=0,
+            NPERIODS=10,
+            ULAMBDA=0.15,
+            K=22.591,
+            ENERGY=6.0,
+            PHOT_ENERGY_MIN=100.0,
+            PHOT_ENERGY_MAX=100100.0,
+            NPOINTS=100,
+            NTRAJPOINTS=101,
+            CURRENT=200.0,
+            FILE="?",
+            SLIT_FLAG=1,
+            SLIT_D=30.0,
+            SLIT_NY=50,
+            SLIT_WIDTH_H_MM=115 * 0.4,
+            SLIT_CENTER_H_MM=0.0,
+            SLIT_HEIGHT_V_MM=60  * 0.3,
+            SLIT_CENTER_V_MM=0.0,
+        )
+
+        #
+        # script to make the calculations (created by XOPPY:WS)
+        #
+        import numpy
+        from xoppylib.xoppy_run_binaries import xoppy_calc_ws
+
+        out_file = xoppy_calc_ws(
+            ENERGY=6.0,
+            CUR=200.0,
+            PERIOD=15.0,
+            N=10.0,
+            KX=0.0,
+            KY=22.591,
+            EMIN=100.0,
+            EMAX=100100.0,
+            NEE=1000,
+            D=30.0,
+            XPC=0.0,
+            YPC=0.0,
+            XPS=115.0 * 0.4,
+            YPS=60.0  * 0.3,
+            NXP=70,
+            NYP=70,
+        )
+
+        # data to pass to power
+        data = numpy.loadtxt(out_file)
+        energy2 = data[:, 0]
+        flux2 = data[:, 1]
+        # spectral_power = data[:, 2]
+        # cumulated_power = data[:, 3]
+
+        #
+        # example plot
+        #
+        from srxraylib.plot.gol import plot
+
+        plot(energy, flux,
+             energy2, flux2,
+             xtitle="Photon energy [eV]", ytitle="Flux [photons/s/0.1%bw]", title="WS Flux",
+             legend=['WIGGLER', 'WS'],
+             xlog=0, ylog=0, show=True)
